@@ -22,12 +22,15 @@ use CPAN::DistnameInfo ();
 {
     package Parse::CPAN::Packages::Fast;
 
-    our $VERSION = '0.05';
+    our $VERSION = '0.06';
+    $VERSION =~ s{_}{};
 
     use PerlIO::gzip;
-    use version;
+    use CPAN::Version ();
 
-    sub _default_packages_file {
+    # Note: this function is possibly interactive, i.e. if CPAN.pm was
+    # never configured, or if CPAN's Config.pm needs reconfiguration.
+    sub _default_packages_file_interactive {
 	my($class) = @_;
 	require CPAN;
 	no warnings 'once';
@@ -37,6 +40,41 @@ use CPAN::DistnameInfo ();
 	$packages_file;
     }
 
+    # Note: this function is guaranteed to be non-interactive, but it
+    # is using just default locations to look at the CPAN config, or
+    # the 02packages files.
+    sub _default_packages_file_batch {
+	my($class) = @_;
+
+	my $home_cpandir = do {
+	    no warnings 'uninitialized'; # HOME may be uninitialized on some systems e.g. Windows
+	    "$ENV{HOME}/.cpan";
+	};
+	if (!$INC{"CPAN/MyConfig.pm"}) {
+	    my $home_myconfig = "$home_cpandir/CPAN/MyConfig.pm";
+	    if (-r $home_myconfig) {
+		local @INC = ($home_cpandir);
+		eval { require "CPAN::MyConfig" };
+	    }
+	}
+	if ($INC{"CPAN/MyConfig.pm"} && $CPAN::Config->{keep_source_where}) {
+	    my $packages_file = $CPAN::Config->{keep_source_where} . "/modules/02packages.details.txt.gz";
+	    if (-r $packages_file && -s $packages_file) {
+		return $packages_file;
+	    }
+	}
+
+	# Cannot find a usable CPAN::MyConfig, try a default location
+	my $packages_file = "$home_cpandir/sources/modules/02packages.details.txt.gz";
+	if (-r $packages_file && -s $packages_file) {
+	    return $packages_file;
+	}
+
+	undef;
+    }
+
+    *_default_packages_file = \&_default_packages_file_interactive;
+
     sub new {
 	my($class, $packages_file) = @_;
 
@@ -108,11 +146,12 @@ use CPAN::DistnameInfo ();
 	my($self, $distribution_name) = @_;
 	my @candidates;
 	for my $candidate (keys %{ $self->{dist_to_pkgs} }) {
-	    if ($candidate =~ m{/\Q$distribution_name}) {
+	    if ($candidate =~ m{^./../.*/\Q$distribution_name}) {
 		# Possibly pure CPAN::DistnameInfo is somewhat faster
 		# than Parse::CPAN::Packages::Fast::Distribution (no
 		# inside-out handling, no additional DESTROY)
 		my $d = CPAN::DistnameInfo->new($candidate);
+		no warnings 'uninitialized'; # Some distributions have no parseable dist name
 		if ($d->dist eq $distribution_name) {
 		    push @candidates, $d;
 		}
@@ -120,10 +159,10 @@ use CPAN::DistnameInfo ();
 	}
 	return if !@candidates; # XXX die or not?
 	my $best_candidate = pop @candidates;
-	my $best_candidate_version = version->new($best_candidate->version);
+	my $best_candidate_version = $best_candidate->version;
 	for my $candidate (@candidates) {
-	    my $this_version = version->new($candidate->version);
-	    if ($best_candidate_version < $this_version) {
+	    my $this_version = $candidate->version;
+	    if (CPAN::Version->vlt($best_candidate_version, $this_version)) {
 		$best_candidate = $candidate;
 		$best_candidate_version = $this_version;
 	    }
@@ -141,8 +180,7 @@ use CPAN::DistnameInfo ();
 	    if (!exists $latest_dist{$dist}) {
 		$latest_dist{$dist} = $d;
 	    } else {
-		no warnings;
-		if (eval { version->new($latest_dist{$dist}->version) < version->new($d->version) }) {
+		if (CPAN::Version->vlt($latest_dist{$dist}->version, $d->version)) {
 		    $latest_dist{$dist} = $d;
 		}
 	    }
@@ -155,6 +193,62 @@ use CPAN::DistnameInfo ();
 	my @dists = $self->latest_distributions;
 	scalar @dists;
     }
+
+    # Addition: fast module lookup without loading whole packages file
+    # Not yet official!
+    sub _module_lookup {
+	die "Usage?" if @_ != 4;
+	my($class, $module, $orig_packages_file, $cache_file) = @_;
+	require Search::Dict;
+	my $pwhfh = $class->_get_plain_packages_fh($orig_packages_file, $cache_file);
+	my $skey = "$module ";
+	return if Search::Dict::look($pwhfh, $skey, 0, 1) == -1;
+	while () {
+	    my $got = <$pwhfh>;
+	    if (index($got, $skey) == 0) {
+		chomp $got;
+		my($pkg, $ver, $dist) = split /\s+/, $got;
+		return {
+			package => $pkg,
+			version => $ver,
+			dist    => $dist,
+		       };
+	    }
+	    return if lc(substr($got, 0, length($skey))) gt lc($skey);
+	    return if eof($pwhfh);
+	}
+    }
+	
+    sub _get_plain_packages_fh {
+	die "Usage?" if @_ != 3;
+	my(undef, $orig_packages_file, $cache_file) = @_;
+	die "$orig_packages_file does not exist" if !-e $orig_packages_file;
+	if (!-e $cache_file || -M $cache_file > -M $orig_packages_file) {
+	    open my $ifh, "<:gzip", $orig_packages_file
+		or die "Can't open $orig_packages_file: $!";
+	    require File::Temp;
+	    require File::Basename;
+	    my($tmpfh,$tmpfile) = File::Temp::tempfile(DIR => File::Basename::dirname($cache_file))
+		or die "Can't create temporary file: $!";
+	    while (<$ifh>) {
+		last if /^$/;
+	    }
+	    {
+		local $/ = \8192;
+		while (<$ifh>) {
+		    print $tmpfh $_;
+		}
+	    }
+	    close $tmpfh
+		or die "Error while writing temporary file $tmpfile: $!";
+	    rename $tmpfile, $cache_file
+		or die "While renaming $tmpfile to $cache_file: $!";
+	}
+	open my $fh, $cache_file
+	    or die "Can't open $cache_file: $!";
+	$fh;
+    }
+	
 }
 
 ######################################################################
@@ -262,9 +356,9 @@ Parse::CPAN::Packages::Fast - parse CPAN's package index
 
     use Parse::CPAN::Packages::Fast;
 
-    my $p = Parse::CPAN::Packages::Fast->new;
-    ## or alternatively, if CPAN.pm is not configured:
-    # my $p = Parse::CPAN::Packages::Fast->new("/path/to/02packages.details.txt.gz");
+    my $p = Parse::CPAN::Packages::Fast->new("/path/to/02packages.details.txt.gz");
+    ## Or alternatively, if CPAN.pm is configured
+    #my $p = Parse::CPAN::Packages::Fast->new;
 
     my $m = $p->package("Kwalify");
     # $m is a Parse::CPAN::Packages::Fast::Package object
@@ -298,6 +392,12 @@ Parse::CPAN::Packages::Fast::Package object does not include the
 containing distribution in the data structure, but it's necessary to
 use the C<distribution> method.
 
+=item * The C<new> constructor may be called without the path to the
+C<02packages.details.txt> file. In this case L<CPAN.pm|CPAN>'s logic
+is used to find an existing packages file. Note that this might be
+interactive (i.e. if CPAN.pm was never configured, or needs
+reconfiguration), so don't do this in batch systems.
+
 =back
 
 =head2 WHY?
